@@ -21,10 +21,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_GNOME
-#include <libgnome/gnome-help.h>
-#endif
-
 #ifdef HAVE_GPGME
 #  include "rfc3156.h"
 #endif
@@ -35,6 +31,12 @@
 #include <glib/gi18n.h>
 #include "misc.h"
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+
 #if ENABLE_ESMTP
 #include <string.h>
 #include "smtp-server.h"
@@ -116,6 +118,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->always_trust = FALSE;
     ident->warn_send_plain = TRUE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
+    ident->force_key_id = NULL;
 #endif
     ident->request_mdn = FALSE;
     /*
@@ -146,6 +149,9 @@ libbalsa_identity_finalize(GObject * object)
 #endif                          /* ENABLE_ESMTP */
     g_free(ident->face);
     g_free(ident->x_face);
+#ifdef HAVE_GPGME
+    g_free(ident->force_key_id);
+#endif
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -979,13 +985,13 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     /* create the "General" tab */
     table = append_ident_notebook_page(notebook, 5, _("General"), NULL);
     row = 0;
-    ident_dialog_add_entry(table, row++, dialog, _("_Identity Name:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Identity name:"), 
 		           "identity-name");
-    ident_dialog_add_entry(table, row++, dialog, _("_Full Name:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Full name:"), 
                            "identity-fullname");
-    ident_dialog_add_entry(table, row++, dialog, _("_Mailing Address:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("_Mailing address:"), 
                            "identity-address");
-    ident_dialog_add_entry(table, row++, dialog, _("Reply _To:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("Reply _to:"), 
                            "identity-replyto");
     ident_dialog_add_entry(table, row++, dialog, _("_Domain:"), 
                            "identity-domain");
@@ -995,9 +1001,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Bcc:"), 
                            "identity-bcc");
-    ident_dialog_add_entry(table, row++, dialog, _("Reply _String:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("Reply _string:"), 
                            "identity-replystring");
-    ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
+    ident_dialog_add_entry(table, row++, dialog, _("F_orward string:"), 
                            "identity-forwardstring");
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("send messages in both plain text and _HTML format"),
@@ -1013,7 +1019,7 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            path_info[LBI_PATH_TYPE_FACE].box_key,
                            path_info[LBI_PATH_TYPE_XFACE].box_key);
 #if ENABLE_ESMTP
-    ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P Server:"),
+    ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P server:"),
                                "identity-smtp-server", smtp_servers);
 #endif /* ENABLE_ESMTP */
 
@@ -1021,25 +1027,25 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     table = append_ident_notebook_page(notebook, 7, _("Signature"), NULL);
     row = 0;
     ident_dialog_add_check_and_entry(table, row++, dialog,
-                                     _("Signature _Path"),
+                                     _("Signature _path"),
                                      "identity-sigpath");
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                _("_Execute Signature"),
+                                _("_Execute signature"),
 				 "identity-sigexecutable", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Incl_ude Signature"), 
+                                 _("Incl_ude signature"), 
                                  "identity-sigappend", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
-                                 _("Include Signature When For_warding"),
+                                 _("Include signature when for_warding"),
                                  "identity-whenforward", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Include Signature When Rep_lying"),
+                                 _("Include signature when rep_lying"),
                                  "identity-whenreply", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
-                                 _("_Add Signature Separator"),
+                                 _("_Add signature separator"),
                                  "identity-sigseparator", FALSE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("Prepend Si_gnature"),
+                                 _("Prepend si_gnature"),
                                  "identity-sigprepend", FALSE);
 
 #ifdef HAVE_GPGME
@@ -1067,6 +1073,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("remind me if messages can be encrypted"),
                                  "identity-warn-send-plain", TRUE);
+    ident_dialog_add_entry(table, row++, dialog,
+                           _("use secret key with this id for signing\n(leave empty for automatic selection)"), 
+                           "identity-keyid");
 #ifndef HAVE_GPGME
     gtk_widget_set_sensitive(table, FALSE);
 #endif
@@ -1511,6 +1520,7 @@ ident_dialog_update(GObject * dlg)
     id->warn_send_plain = ident_dialog_get_bool(dlg, "identity-warn-send-plain");
     id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
                                           (dlg, "identity-crypt-protocol"));
+    id->force_key_id    = g_strstrip(ident_dialog_get_text(dlg, "identity-keyid"));
 #endif
    
     return TRUE;
@@ -1678,17 +1688,22 @@ delete_ident_response(GtkWidget * confirm, gint response,
 static void
 help_ident_cb(void)
 {
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err = NULL;
 
-#ifdef HAVE_GNOME
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gtk_show_uri(NULL, "ghelp:balsa?identities",
+                 gtk_get_current_event_time(), &err);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
     gnome_help_display("balsa", "identities", &err);
-#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 
     if (err) {
         g_print(_("Error displaying help for identities: %s\n"),
                 err->message);
         g_error_free(err);
     }
+#endif
 }
 
 /* libbalsa_identity_config_dialog displays an identity management
@@ -1895,6 +1910,7 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                               ident->warn_send_plain);    
     display_frame_set_gpg_mode(dialog, "identity-crypt-protocol",
 			   &ident->crypt_protocol);
+    display_frame_set_field(dialog, "identity-keyid", ident->force_key_id);
 #endif
 }
 
@@ -1997,6 +2013,7 @@ libbalsa_identity_new_config(const gchar* name)
     ident->always_trust = libbalsa_conf_get_bool("GpgTrustAlways");
     ident->warn_send_plain = libbalsa_conf_get_bool("GpgWarnSendPlain=true");
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
+    ident->force_key_id = libbalsa_conf_get_string("ForceKeyID");
 #endif
 
     return ident;
@@ -2044,6 +2061,7 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_bool("GpgTrustAlways", ident->always_trust);
     libbalsa_conf_set_bool("GpgWarnSendPlain", ident->warn_send_plain);
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
+    libbalsa_conf_set_string("ForceKeyID", ident->force_key_id);
 #endif
 
     libbalsa_conf_pop_group();
