@@ -224,7 +224,12 @@ struct _LibBalsaMailbox {
     LibBalsaCondition *view_filter; /* to choose a subset of messages
                                      * to be displayed, e.g., only
                                      * undeleted. */
-    LibBalsaCondition *persistent_view_filter;
+    LibBalsaCondition *persistent_view_filter; /* the part of the view 
+                                                * filter that will persist 
+                                                * to the next time the
+                                                * mailbox is opened */
+    gboolean view_filter_pending;  /* a view filter has been set
+                                    * but the view has not been updated */
 
     /* info fields */
     gboolean has_unread_messages;
@@ -259,6 +264,13 @@ struct _LibBalsaMailboxSearchIter {
     gpointer user_data;		/* private backend info */
 };
 
+/** Iterates over a list of messages, returning each time it is called
+    flags and the stream to a message. It is the responsibility of the
+    called to un-ref the stream after use. */
+typedef gboolean (*LibBalsaAddMessageIterator)(LibBalsaMessageFlag *,
+					       GMimeStream **stream,
+					       void *);
+
 struct _LibBalsaMailboxClass {
     GObjectClass parent_class;
 
@@ -284,7 +296,7 @@ struct _LibBalsaMailboxClass {
 				  LibBalsaMessageBody *part,
                                   GError **err);
     GMimeStream *(*get_message_stream) (LibBalsaMailbox * mailbox,
-                                        guint msgno);
+                                        guint msgno, gboolean peek);
 
     void (*check) (LibBalsaMailbox * mailbox);
 
@@ -297,9 +309,9 @@ struct _LibBalsaMailboxClass {
     void (*save_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     void (*load_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     gboolean (*sync) (LibBalsaMailbox * mailbox, gboolean expunge);
-    gboolean (*add_message) (LibBalsaMailbox * mailbox,
-                             GMimeStream * stream,
-                             LibBalsaMessageFlag flags, GError ** err);
+    guint (*add_messages) (LibBalsaMailbox * mailbox,
+			   LibBalsaAddMessageIterator msg_iterator,
+			   void *iter_arg, GError ** err);
     gboolean (*messages_change_flags) (LibBalsaMailbox * mailbox,
 				       GArray *msgnos,
 				       LibBalsaMessageFlag set,
@@ -400,7 +412,7 @@ gboolean libbalsa_mailbox_get_message_part(LibBalsaMessage    *message,
     with full RFC822 text of the message.
 */
 GMimeStream *libbalsa_mailbox_get_message_stream(LibBalsaMailbox * mailbox,
-						 guint msgno);
+						 guint msgno, gboolean peek);
 
 
 /** libbalsa_mailbox_sync_storage() asks the mailbox to synchronise
@@ -449,6 +461,12 @@ gboolean libbalsa_mailbox_add_message(LibBalsaMailbox * mailbox,
                                       GMimeStream * stream,
                                       LibBalsaMessageFlag flags,
                                       GError ** err);
+
+guint libbalsa_mailbox_add_messages(LibBalsaMailbox * mailbox,
+				    LibBalsaAddMessageIterator msg_iterator,
+				    void *arg,
+				    GError ** err);
+
 gboolean libbalsa_mailbox_close_backend(LibBalsaMailbox * mailbox);
 
 /* Message number-list methods */
@@ -470,7 +488,7 @@ GType libbalsa_mailbox_type_from_path(const gchar * filename);
 
 guint libbalsa_mailbox_total_messages(LibBalsaMailbox * mailbox);
 gboolean libbalsa_mailbox_can_move_duplicates(LibBalsaMailbox * mailbox);
-void libbalsa_mailbox_move_duplicates(LibBalsaMailbox * mailbox,
+gint libbalsa_mailbox_move_duplicates(LibBalsaMailbox * mailbox,
                                       LibBalsaMailbox * dest,
                                       GError ** err);
 
@@ -482,7 +500,8 @@ LibBalsaMailboxIndexEntry* libbalsa_mailbox_index_entry_new_from_msg
                            (LibBalsaMessage *msg);
 void libbalsa_mailbox_index_entry_set_no(LibBalsaMailboxIndexEntry *entry,
                                          unsigned no);
-void libbalsa_mailbox_index_entry_free(LibBalsaMailboxIndexEntry *entry);
+void libbalsa_mailbox_index_entry_clear(LibBalsaMailbox * mailbox,
+                                        guint msgno);
 void libbalsa_mailbox_index_set_flags(LibBalsaMailbox *mailbox,
 				      unsigned msgno, LibBalsaMessageFlag f);
 gboolean libbalsa_mailbox_set_view_filter(LibBalsaMailbox * mailbox,
