@@ -26,6 +26,20 @@
 #include "imap/libimap.h"
 #include "libbalsa.h"
 
+#if defined (HAVE_GNOME_KEYRING)
+#include <gnome-keyring.h>
+
+#if defined(HAVE_GNOME_KEYRING_24)
+#define LIBBALSA_SERVER_KEYRING_SCHEMA GNOME_KEYRING_NETWORK_PASSWORD
+#else
+extern const GnomeKeyringPasswordSchema* LIBBALSA_SERVER_KEYRING_SCHEMA;
+#endif /* HAVE_GNOME_KEYRING_24 */
+
+#define libbalsa_free_password gnome_keyring_free_password
+#else
+#define libbalsa_free_password g_free
+#endif /* HAVE_GNOME_KEYRING */
+
 #define LIBBALSA_TYPE_SERVER \
     (libbalsa_server_get_type())
 #define LIBBALSA_SERVER(obj) \
@@ -50,6 +64,7 @@ typedef enum {
 
 struct _LibBalsaServer {
     GObject object;
+    const gchar *protocol; /**< type of the server: imap, pop3, or smtp. */
 
     gchar *host;
     gchar *user;
@@ -93,4 +108,5 @@ void libbalsa_server_user_cb(ImapUserEventType ue, void *arg, ...);
 
 void libbalsa_server_connect_signals(LibBalsaServer * server, GCallback cb,
                                      gpointer cb_data);
+
 #endif				/* __LIBBALSA_SERVER_H__ */
