@@ -28,10 +28,6 @@
 #include <string.h>
 #include <libesmtp.h>
 
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif                          /* HAVE_GNOME */
-
 #include "libbalsa.h"
 #include "server.h"
 #include "smtp-server.h"
@@ -39,6 +35,12 @@
 #include "misc.h"
 #include <glib/gi18n.h>
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+
 static LibBalsaServerClass *parent_class = NULL;
 
 struct _LibBalsaSmtpServer {
@@ -105,8 +107,15 @@ authinteract(auth_client_request_t request, char **result, int fields,
     int i;
 
     for (i = 0; i < fields; i++) {
-        if (request[i].flags & AUTH_PASS)
+        if (request[i].flags & AUTH_PASS) {
+	    /* We need to return a const pointer, this is why we
+	       ignore the result from get_password, and take and
+	       advantage of the fact that this function sets the
+	       passwd field of the server. */
+	    if(!server->passwd)
+		g_free(libbalsa_server_get_password(server, NULL));
             result[i] = server->passwd;
+	}
         else if (request[i].flags & AUTH_USER)
             result[i] = (server->user
                          && *server->user) ? server->user : NULL;
@@ -140,6 +149,7 @@ tlsinteract(char *buf, int buflen, int rwflag, void *arg)
 static void
 libbalsa_smtp_server_init(LibBalsaSmtpServer * smtp_server)
 {
+    LIBBALSA_SERVER(smtp_server)->protocol = "smtp";
     smtp_server->authctx = auth_create_context();
     auth_set_mechanism_flags(smtp_server->authctx, AUTH_PLUGIN_PLAIN, 0);
     auth_set_interact_cb(smtp_server->authctx, authinteract, smtp_server);
@@ -200,6 +210,9 @@ libbalsa_smtp_server_new(void)
 
     smtp_server = g_object_new(LIBBALSA_TYPE_SMTP_SERVER, NULL);
 
+    /* Change the default. */
+    LIBBALSA_SERVER(smtp_server)->remember_passwd = TRUE;
+
     return smtp_server;
 }
 
@@ -231,8 +244,6 @@ libbalsa_smtp_server_new_from_config(const gchar * name)
 void
 libbalsa_smtp_server_save_config(LibBalsaSmtpServer * smtp_server)
 {
-    /* FIXME: isn't it a bit of a brute force? */
-    LIBBALSA_SERVER(smtp_server)->remember_passwd = TRUE;
     libbalsa_server_save_config(LIBBALSA_SERVER(smtp_server));
 
 #if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
@@ -396,28 +407,31 @@ smtp_server_tls_widget(LibBalsaSmtpServer * smtp_server)
 }
 #endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
-static const gchar smtp_server_section[] = "smtp-server-config";
-
 static void
 smtp_server_response(GtkDialog * dialog, gint response,
                      struct smtp_server_dialog_info *sdi)
 {
     LibBalsaServer *server = LIBBALSA_SERVER(sdi->smtp_server);
-#ifdef HAVE_GNOME
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *error = NULL;
-#endif                          /* HAVE_GNOME */
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 
     switch (response) {
     case GTK_RESPONSE_HELP:
-#ifdef HAVE_GNOME
-        gnome_help_display("balsa", smtp_server_section, &error);
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, "ghelp:balsa?smtp-server-config",
+                     gtk_get_current_event_time(), &error);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
+        gnome_help_display("balsa", "smtp-server-config", &error);
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
         if (error) {
             libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                 _("Error displaying %s: %s\n"),
-                                 smtp_server_section, error->message);
+                                 _("Error displaying server help: %s\n"),
+                                 error->message);
             g_error_free(error);
         }
-#endif                          /* HAVE_GNOME */
+#endif
         return;
     case GTK_RESPONSE_OK:
         libbalsa_smtp_server_set_name(sdi->smtp_server,
