@@ -46,14 +46,19 @@ struct EntryMaster_s {
 #define ENTRY_MASTER_P_DONE( ep ) ( ((ep)->setbits & (ep)->donemask) == (ep)->donemask )
 #define ENTRY_MASTER_DONE( e ) ( ((e).setbits & (e).donemask) == (e).donemask )
 
-#if BALSA_MAJOR < 2
-GdkImlibImage *balsa_init_get_png(const gchar * fname);
-#else
 GdkPixbuf *balsa_init_get_png(const gchar * fname);
-#endif                          /* BALSA_MAJOR < 2 */
+
 void balsa_init_add_table_entry(GtkTable * table, guint num, gchar * ltext,
                                 const gchar * etext, EntryData * ed,
                                 GnomeDruid * druid, GtkWidget ** dest);
+void balsa_init_add_table_option(GtkTable *table, guint num,
+                                 const gchar *ltext, const gchar **optns,
+                                 GnomeDruid *druid, GtkWidget **dest);
+void balsa_init_add_table_checkbox(GtkTable *table, guint num,
+                                   const gchar *ltext, gboolean defval,
+                                   GnomeDruid *druid, GtkWidget **dest);
+gint balsa_option_get_active(GtkWidget *option_widget);
+
 gboolean balsa_init_create_to_directory(const gchar * dir,
                                         gchar ** complaint);
 
