@@ -22,7 +22,10 @@
 #include "config.h"
 
 #include <string.h>
+#include <gtk/gtk.h>
+#if HAVE_GNOME
 #include <gnome.h>
+#endif
 #ifdef GTKHTML_HAVE_GCONF
 # include <gconf/gconf.h>
 #endif
@@ -1053,7 +1056,8 @@ ab_warning(const char *fmt, ...)
     va_end(va_args);
     d = gtk_message_dialog_new(GTK_WINDOW(ab_window),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
-                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg);
+                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                               "%s", msg);
     gtk_dialog_run(GTK_DIALOG(d));
     gtk_widget_destroy(d);
 }
@@ -1176,7 +1180,9 @@ main(int argc, char *argv[])
 static void
 bab_cleanup(void)
 {
+#if HAVE_GNOME
     gnome_sound_shutdown();
+#endif
     gtk_main_quit();
 }
 
