@@ -20,7 +20,6 @@
  */
 
 #include <string.h>
-#include <gnome.h>
 #include <gdk/gdkx.h>
 
 #include "config.h"
@@ -83,6 +82,11 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	{ "user-trash-full", GTK_STOCK_DELETE },
 	{ "mail-attachment", "stock_attach" },
 	{ "emblem-important", "stock_mail-flag-for-followup"},
+        { "stock_mail-reply", "stock_mail-replied"}, 
+        { "stock_mail-reply-to-all", "stock_mail-replied"}, 
+        { "stock_mail-forward", "stock_mail-replied"}, 
+        { "gnome-fs-directory-accept", GTK_STOCK_OPEN},
+        { "gnome-fs-directory", GTK_STOCK_DIRECTORY},
 	{ NULL, NULL } };
 
     BICONS_LOG("loading icon %s (stock id %s)", bpixmap->name,
@@ -94,11 +98,13 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	pixmap_fallback_t *fb = fallback_id;
 	while (fb->def_id && strcmp(fb->def_id, bpixmap->stock_id))
 	    fb++;
-	if (fb->def_id)
+	if (fb->def_id) {
 	    use_id = fb->fb_id;
-	else {
+            BICONS_LOG("\t(%s not found, fall back to %s)",
+                       bpixmap->stock_id, use_id);
+        } else {
 	    BICONS_ERR("icon %s unknown, no fallback", bpixmap->stock_id);
-	    return;
+	    use_id = GTK_STOCK_MISSING_IMAGE;
 	}
     } else
 	use_id = bpixmap->stock_id;
@@ -147,6 +153,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 		gtk_icon_source_set_size(icon_source, bpixmap->sizes[n]);
 		gtk_icon_source_set_size_wildcarded(icon_source, FALSE);
 		gtk_icon_set_add_source(icon_set, icon_source);
+                gtk_icon_source_free(icon_source);
 	    }
 	} else
 	    BICONS_ERR("bad size %d", bpixmap->sizes[n]);
@@ -166,6 +173,8 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_REPLY_GROUP,     "stock_mail-reply-to-all", 
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
+	{ BALSA_PIXMAP_REQUEST_MDN,     "stock_mail-reply", 
+	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_SEND,            "stock_mail-send",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_RECEIVE,         "stock_mail-receive",
