@@ -21,7 +21,6 @@
 
 #include "config.h"
 
-#include <gnome.h>
 /* #include <gtk/gtkfeatures.h> */
 #include <string.h>
 #include <gdk/gdkfont.h>
@@ -139,10 +138,10 @@ static void bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path);
 
 /* class methods */
 
-GtkType
+GType
 balsa_mblist_get_type(void)
 {
-    static GtkType mblist_type = 0;
+    static GType mblist_type = 0;
 
     if (!mblist_type) {
 	static const GTypeInfo mblist_info = {
@@ -1154,6 +1153,10 @@ balsa_mblist_find_all_unread_mboxes(LibBalsaMailbox * mailbox)
     info.mailbox = mailbox;
     info.list = NULL;
 
+    if (!balsa_app.mblist_tree_store) /* We have no mailboxes, maybe
+					 we are about to quit? */
+	return NULL;
+
     gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
                            bmbl_find_all_unread_mboxes_func, &info);
 
