@@ -20,6 +20,7 @@
  */
 
 #include <string.h>
+#include <stdlib.h>
 #include "config.h"
 #include "balsa-app.h"
 #include "html.h"
@@ -32,6 +33,10 @@
 #include "balsa-mime-widget-text.h"
 #include "balsa-cite-bar.h"
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#include <libgnome/gnome-url.h>
+#endif
+
 #if HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
@@ -99,12 +104,13 @@ static gboolean store_button_coords(GtkWidget * widget, GdkEventButton * event,
 static gboolean check_over_url(GtkWidget * widget, GdkEventMotion * event, GList * url_list);
 static void pointer_over_url(GtkWidget * widget, message_url_t * url, gboolean set);
 static void prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list);
-static void url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter, const gchar * buf, gpointer data);
+static void url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
+                         const gchar * buf, guint len, gpointer data);
 static gboolean check_call_url(GtkWidget * widget, GdkEventButton * event, GList * url_list);
 static message_url_t * find_url(GtkWidget * widget, gint x, gint y, GList * url_list);
 static void handle_url(const message_url_t* url);
 static void free_url_list(GList * url_list);
-static void balsa_gtk_html_on_url(GtkWidget *html, const gchar *url);
+static void bm_widget_on_url(const gchar *url);
 static void phrase_highlight(GtkTextBuffer * buffer, const gchar * id,
 			     gunichar tag_char, const gchar * property,
 			     gint value);
@@ -124,7 +130,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 {
     LibBalsaHTMLType html_type;
     gchar *ptr = NULL;
-    size_t alloced;
+    ssize_t alloced;
     BalsaMimeWidget *mw;
     GtkTextBuffer *buffer;
 #if USE_GREGEX
@@ -143,7 +149,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 
     is_text_plain = !g_ascii_strcasecmp(content_type, "text/plain");
     alloced = libbalsa_message_body_get_content(mime_body, &ptr, &err);
-    if (!ptr) {
+    if (alloced < 0) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
                           _("Could not save a text part: %s"),
                           err ? err->message : "Unknown error");
@@ -168,8 +174,13 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
         return NULL;
 #endif
     }
-    if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0) {
-        return bm_widget_new_vcard(bm, mime_body, ptr, alloced);
+    if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0 ||
+       g_ascii_strcasecmp(content_type, "text/directory") == 0) {
+        mw = bm_widget_new_vcard(bm, mime_body, ptr, alloced);
+        if (mw)
+            return mw;
+        /* else it was not a vCard with at least one address; we'll just
+         * show it as if it were text/plain. */
     }
 
     /* prepare a text part */
@@ -215,8 +226,6 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	       )
 	libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
 
-    mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-    mw->widget = create_text_widget(content_type);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(mw->widget), FALSE);
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(mw->widget), 2);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(mw->widget), 15);
@@ -233,7 +242,6 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		     (gpointer)mime_body);
 
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->widget));
-    gtk_text_buffer_create_tag(buffer, "soft", NULL, NULL);
     allocate_quote_colors(GTK_WIDGET(bm), balsa_app.quoted_color,
 			  0, MAX_QUOTED_COLOR - 1);
 #if USE_GREGEX
@@ -247,7 +255,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     }
 #endif                          /* USE_GREGEX */
     else {
-	gchar *line_start;
+	const gchar *line_start;
 	LibBalsaUrlInsertInfo url_info;
 	GList * cite_bars_list;
 	guint cite_level;
@@ -269,23 +277,19 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	url_info.callback = url_found_cb;
 	url_info.callback_data = &url_list;
 	url_info.buffer_is_flowed = libbalsa_message_body_is_flowed(mime_body);
-	url_info.ml_url = NULL;
 	url_info.ml_url_buffer = NULL;
 
 	line_start = ptr;
 	cite_bars_list = NULL;
 	cite_level = 0;
 	cite_start = 0;
-	while (line_start) {
-	    gchar *newline, *this_line;
+	while (*line_start) {
+	    const gchar *line_end;
 	    GtkTextTag *tag = NULL;
 	    int len;
 
-	    newline = strchr(line_start, '\n');
-	    if (newline)
-		this_line = g_strndup(line_start, newline - line_start);
-	    else
-		this_line = g_strdup(line_start);
+	    if (!(line_end = strchr(line_start, '\n')))
+                line_end = line_start + strlen(line_start);
 
 	    if (is_text_plain) {
 		guint quote_level;
@@ -293,9 +297,11 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 
 		/* get the cite level only for text/plain parts */
 #if USE_GREGEX
-		libbalsa_match_regex(this_line, rex, &quote_level, &cite_idx);
+		libbalsa_match_regex(line_start, rex, &quote_level,
+                                     &cite_idx);
 #else                           /* USE_GREGEX */
-		libbalsa_match_regex(this_line, &rex, &quote_level, &cite_idx);
+		libbalsa_match_regex(line_start, &rex, &quote_level,
+                                     &cite_idx);
 #endif                          /* USE_GREGEX */
 	    
 		/* check if the citation level changed */
@@ -313,31 +319,26 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		    cite_level = quote_level;
 		}
 
-		/* strip off the citation prefix */
+		/* skip the citation prefix */
 		if (quote_level) {
-		    gchar *new;
-		
-		    new = this_line + cite_idx;
-		    if (g_unichar_isspace(g_utf8_get_char(new)))
-			new = g_utf8_next_char(new);
-		    new = g_strdup(new);
-		    g_free(this_line);
-		    this_line = new;
+		    line_start += cite_idx;
+                    if (line_start < line_end
+                        && g_unichar_isspace(g_utf8_get_char(line_start)))
+                        line_start = g_utf8_next_char(line_start);
 		}
 		tag = quote_tag(buffer, quote_level, margin);
 	    }
 
-	    len = strlen(this_line);
-	    if (len > 0 && this_line[len - 1] == '\r')
-		this_line[len - 1] = '\0';
+            len = line_end - line_start;
+	    if (len > 0 && line_start[len - 1] == '\r')
+                --len;
 	    /* tag is NULL if the line isn't quoted, but it causes
 	     * no harm */
-	    if (!libbalsa_insert_with_url(buffer, this_line, line_start,
+	    if (!libbalsa_insert_with_url(buffer, line_start, len,
 					  tag, &url_info))
 		gtk_text_buffer_insert_at_cursor(buffer, "\n", 1);
 	    
-	    g_free(this_line);
-	    line_start = newline ? newline + 1 : NULL;
+	    line_start = *line_end ? line_end + 1 : line_end;
 	}
 
 	/* add any pending cited part */
@@ -677,8 +678,8 @@ text_view_populate_popup(GtkTextView *textview, GtkMenu *menu,
                           (GtkCallback)gtk_widget_destroy_insensitive, NULL);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu),
 			  gtk_separator_menu_item_new ());
-    libbalsa_fill_vfs_menu_by_content_type(menu, "text/plain",
-					   G_CALLBACK (balsa_mime_widget_ctx_menu_vfs_cb),
+    libbalsa_vfs_fill_menu_by_content_type(menu, "text/plain",
+					   G_CALLBACK (balsa_mime_widget_ctx_menu_cb),
 					   (gpointer)mime_body);
 
     menu_item = gtk_menu_item_new_with_label (_("Save..."));
@@ -795,10 +796,10 @@ pointer_over_url(GtkWidget * widget, message_url_t * url, gboolean set)
     
     if (set) {
         gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
-        balsa_gtk_html_on_url(NULL, url->url);
+        bm_widget_on_url(url->url);
     } else {
         gtk_text_buffer_remove_tag(buffer, tag, &start, &end);
-        balsa_gtk_html_on_url(NULL, NULL);
+        bm_widget_on_url(NULL);
     }
 }
 
@@ -829,7 +830,7 @@ prepare_url_offsets(GtkTextBuffer * buffer, GList * url_list)
 
 static void
 url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
-             const gchar * buf, gpointer data)
+             const gchar * buf, guint len, gpointer data)
 {
     GList **url_list = data;
     message_url_t *url_found;
@@ -837,7 +838,7 @@ url_found_cb(GtkTextBuffer * buffer, GtkTextIter * iter,
     url_found = g_new(message_url_t, 1);
     url_found->end_mark =
         gtk_text_buffer_create_mark(buffer, NULL, iter, TRUE);
-    url_found->url = g_strdup(buf);       /* gets freed later... */
+    url_found->url = g_strndup(buf, len);       /* gets freed later... */
     *url_list = g_list_append(*url_list, url_found);
 }
 
@@ -927,7 +928,11 @@ handle_url(const message_url_t* url)
         gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
-        gnome_url_show(url->url, &err);
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, url->url, gtk_get_current_event_time(), &err);
+#else
+    gnome_url_show(url->url, &err);
+#endif
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                     _("Error showing %s: %s\n"), url->url,
@@ -1103,7 +1108,7 @@ draw_cite_bars(GtkWidget * widget, GdkEventExpose *event, GList * cite_bars)
 
 /* --- HTML related functions -- */
 static void
-balsa_gtk_html_on_url(GtkWidget *html, const gchar *url)
+bm_widget_on_url(const gchar *url)
 {
     GtkStatusbar *statusbar;
     guint context_id;
@@ -1163,8 +1168,8 @@ balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
     menuitem = gtk_separator_menu_item_new ();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    libbalsa_fill_vfs_menu_by_content_type(GTK_MENU(menu), "text/html",
-					   G_CALLBACK (balsa_mime_widget_ctx_menu_vfs_cb),
+    libbalsa_vfs_fill_menu_by_content_type(GTK_MENU(menu), "text/html",
+					   G_CALLBACK (balsa_mime_widget_ctx_menu_cb),
 					   mime_body);
 
     menuitem = gtk_menu_item_new_with_label (_("Save..."));
@@ -1193,26 +1198,16 @@ balsa_gtk_html_button_press_cb(GtkWidget * html, GdkEventButton * event,
             ? balsa_gtk_html_popup(html, bm) : FALSE);
 }
 
-/* balsa_gtk_html_size_request:
-   report the requested size of the HTML widget.
-*/
 static void
-balsa_gtk_html_size_request(GtkWidget * widget,
-                            GtkRequisition * requisition, gpointer data)
-{
-    g_return_if_fail(widget != NULL);
-    g_return_if_fail(requisition != NULL);
-
-    requisition->width  = GTK_LAYOUT(widget)->hadjustment->upper;
-    requisition->height = GTK_LAYOUT(widget)->vadjustment->upper;
-}
-
-static void
-balsa_gtk_html_link_clicked(GObject *obj, const gchar *url)
+bm_widget_link_clicked(const gchar *url)
 {
     GError *err = NULL;
 
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gtk_show_uri(NULL, url, gtk_get_current_event_time(), &err);
+#else
     gnome_url_show(url, &err);
+#endif
     if (err) {
         balsa_information(LIBBALSA_INFORMATION_WARNING,
                 _("Error showing %s: %s\n"), url, err->message);
@@ -1229,13 +1224,12 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * p
         libbalsa_html_new(ptr, len,
 			  libbalsa_message_body_charset(mime_body),
 			  bm->message,
-                          G_CALLBACK(balsa_gtk_html_link_clicked));
+                          (LibBalsaHtmlCallback)
+                          bm_widget_on_url,
+                          (LibBalsaHtmlCallback)
+                          bm_widget_link_clicked);
     g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
 
-    g_signal_connect(G_OBJECT(mw->widget), "size-request",
-                     G_CALLBACK(balsa_gtk_html_size_request), bm);
-    g_signal_connect(G_OBJECT(mw->widget), "on-url",
-                     G_CALLBACK(balsa_gtk_html_on_url), bm);
     g_signal_connect(G_OBJECT(mw->widget), "button-press-event",
                      G_CALLBACK(balsa_gtk_html_button_press_cb), bm);
     g_signal_connect(G_OBJECT(mw->widget), "key_press_event",
@@ -1258,7 +1252,7 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * p
         row++;                                                    \
     }
 
-BalsaMimeWidget *
+static BalsaMimeWidget *
 bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
                     gchar *ptr, size_t len)
 {
@@ -1269,6 +1263,9 @@ bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
     GtkWidget *w;
     int row = 1;
 
+    if (!addr)
+        return NULL;
+
     mw->widget = gtk_table_new(10, 2, FALSE);
     table = (GtkTable*)mw->widget;
         
