@@ -31,7 +31,6 @@
 #  include <pthread.h>
 #endif
 #include <time.h>
-#include <gnome.h>
 
 #include <string.h>
 #include "filter.h"
@@ -44,6 +43,12 @@
 #include <glib/gi18n.h>
 #include "libbalsa-conf.h"
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
 extern GList * fe_user_headers_list;
@@ -815,7 +820,9 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                           gpointer throwaway)
 {
     LibBalsaCondition *new_cnd;
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err;
+#endif
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -871,8 +878,14 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
 	err = NULL;
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, "ghelp:balsa?win-condition",
+                     gtk_get_current_event_time(), &err);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
 	gnome_help_display("balsa", "win-condition", &err);
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -880,6 +893,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
 		    err->message);
 	    g_error_free(err);
 	}
+#endif
 	break;
     }
     gtk_widget_set_sensitive(fe_window, TRUE);
@@ -982,7 +996,7 @@ build_type_notebook()
 
     box = gtk_vbox_new(FALSE, 5);
     table = gtk_table_new(7, 2, FALSE);
-    gtk_box_pack_start_defaults(GTK_BOX(box), table);
+    gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);
 
     fe_type_simple_label = 
 	gtk_label_new_with_mnemonic(_("One of the specified f_ields contains"));
@@ -1017,7 +1031,7 @@ build_type_notebook()
 
 #if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     page = gtk_table_new(5, 6, FALSE);
-    gtk_box_pack_start_defaults(GTK_BOX(box), page);
+    gtk_box_pack_start(GTK_BOX(box), page, TRUE, TRUE, 0);
 
     fe_type_regex_label = 
         gtk_label_new_with_mnemonic(_("_One of the regular expressions matches"));
@@ -1067,7 +1081,7 @@ build_type_notebook()
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
 #else                   /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
     msg = _("Filtering using regular expressions is not yet implemented.");
-    gtk_box_pack_start_defaults(GTK_BOX(box), gtk_label_new(msg));
+    gtk_box_pack_start(GTK_BOX(box), gtk_label_new(msg), TRUE, TRUE, 0);
 #endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
     /* The date page of the notebook */
@@ -1435,7 +1449,9 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         gtk_tree_view_get_model(fe_filters_list);
     GtkTreeIter iter;
     gboolean valid;
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err;
+#endif
     
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1474,8 +1490,14 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
 	err = NULL;
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, "ghelp:balsa?win-filters",
+                     gtk_get_current_event_time(), &err);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
 	gnome_help_display("balsa", "win-filters", &err);
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1483,6 +1505,7 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
 		    err->message);
 	    g_error_free(err);
 	}
+#endif
 	break;
 
     default:
