@@ -20,7 +20,6 @@
  */
 
 #include "config.h"
-#include <gnome.h>
 #include <string.h>
 #include "balsa-app.h"
 #include "balsa-icons.h"
@@ -32,6 +31,12 @@
 #include "imap-server.h"
 #include <glib/gi18n.h>
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+
 typedef struct _CommonDialogData CommonDialogData;
 typedef struct _FolderDialogData FolderDialogData;
 typedef struct _SubfolderDialogData SubfolderDialogData;
@@ -83,13 +88,13 @@ folder_conf_destroy_cdd(CommonDialogData * cdd)
         g_free(cdd);
 }
 
-static const gchar folder_config_section[] = "folder-config";
-
 static void
 folder_conf_response(GtkDialog * dialog, int response,
                      CommonDialogData * cdd)
 {
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err = NULL;
+#endif
 
     /* If mbnode's parent gets rescanned, mbnode will be finalized,
      * which triggers folder_conf_destroy_cdd, and recursively calls
@@ -99,13 +104,20 @@ folder_conf_response(GtkDialog * dialog, int response,
 	g_object_ref(cdd->mbnode);
     switch (response) {
     case GTK_RESPONSE_HELP:
-        gnome_help_display("balsa", folder_config_section, &err);
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, "ghelp:balsa?folder-config",
+                     gtk_get_current_event_time(), &err);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
+        gnome_help_display("balsa", "folder-config", &err);
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
-		    _("Error displaying %s: %s\n"), folder_config_section,
-                    err->message);
+                              _("Error displaying config help: %s\n"),
+                              err->message);
             g_error_free(err);
         }
+#endif
 	if (cdd->mbnode)
 	    g_object_unref(cdd->mbnode);
         return;
@@ -266,6 +278,14 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     gchar *default_server;
     int r = 0;
 
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
+
     /* Allow only one dialog per mailbox node, and one with mn == NULL
      * for creating a new folder. */
     fcw = mn ? g_object_get_data(G_OBJECT(mn), BALSA_FOLDER_CONF_IMAP_KEY)
@@ -351,7 +371,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                              gtk_label_new_with_mnemonic(_("_Advanced")));
 
     /* INPUT FIELD CREATION */
-    label = libbalsa_create_label(_("Descriptive _Name:"), table, 0);
+    label = libbalsa_create_label(_("Descriptive _name:"), table, 0);
     fcw->folder_name = libbalsa_create_entry(table,
                                    G_CALLBACK(validate_folder),
                                    fcw, r++, mn ? mn->name : NULL, 
@@ -381,7 +401,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
                                 table, r++, s ? s->try_anonymous : FALSE);
     g_signal_connect(G_OBJECT(fcw->anonymous), "toggled",
                      G_CALLBACK(anonymous_cb), fcw);
-    fcw->remember = libbalsa_create_check(_("_Remember password"), 
+    fcw->remember = libbalsa_create_check(_(remember_password_message), 
                                 table, r++, s ? s->remember_passwd : TRUE);
     g_signal_connect(G_OBJECT(fcw->remember), "toggled",
                      G_CALLBACK(remember_cb), fcw);
