@@ -41,7 +41,10 @@
 */
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#if !GTK_CHECK_VERSION(2, 6, 0) && HAVE_GNOME
+#include <gnome.h>  /* needed by GnomeFileEntry */
+#endif
 #include <string.h>
 
 #include "balsa-app.h"
@@ -75,12 +78,12 @@ struct _MailboxConfWindow {
     void (*ok_handler)(MailboxConfWindow*);
     const gchar *ok_button_name;
     GtkWidget *mailbox_name;
-    GtkType mailbox_type;
+    GType mailbox_type;
     BalsaMailboxConfView *view_info;
     gboolean ok_sensitive;
 
     union {
-#if !GTK_CHECK_VERSION(2, 6, 0)
+#if !GTK_CHECK_VERSION(2, 6, 0) && HAVE_GNOME
 	/* for local mailboxes */
 	struct local { 
 	    GnomeFileEntry *path;
@@ -532,7 +535,7 @@ conf_response_cb(GtkDialog* dialog, gint response, MailboxConfWindow * mcw)
 }
 
 static GtkWidget *
-run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type, 
+run_mailbox_conf(BalsaMailboxNode* mbnode, GType mailbox_type, 
 		 gboolean update)
 {
     MailboxConfWindow* mcw;
@@ -575,7 +578,7 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
  * If the used clicks save add the new mailbox to the tree.
  */
 void
-mailbox_conf_new(GtkType mailbox_type)
+mailbox_conf_new(GType mailbox_type)
 {
     static GtkWidget *dialog;
 
@@ -1092,7 +1095,7 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
         label = libbalsa_create_label(_("_Mailbox Name:"), table, ++row);
         mcw->mailbox_name = 
             libbalsa_create_entry(table,
-                         GTK_SIGNAL_FUNC(check_for_blank_fields),
+                         G_CALLBACK(check_for_blank_fields),
                          mcw, row, NULL, label);
     } else mcw->mailbox_name = NULL;
 
@@ -1155,14 +1158,14 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = libbalsa_create_label(_("Mailbox _Name:"), table, 0);
+    label = libbalsa_create_label(_("Mailbox _name:"), table, 0);
     mcw->mailbox_name = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 0, NULL, label);
     /* pop server */
     label = libbalsa_create_label(_("_Server:"), table, 1);
     mcw->mb_data.pop3.bsc.server = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 1, "localhost", label);
     mcw->mb_data.pop3.bsc.default_ports = POP3_DEFAULT_PORTS;
 
@@ -1170,7 +1173,7 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     /* username  */
     label= libbalsa_create_label(_("Use_r name:"), table, 2);
     mcw->mb_data.pop3.username = libbalsa_create_entry(table,
-				     GTK_SIGNAL_FUNC(check_for_blank_fields),
+				     G_CALLBACK(check_for_blank_fields),
 				     mcw, 2, g_get_user_name(), label);
 
     /* password field */
@@ -1199,7 +1202,7 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     label = libbalsa_create_label(_("Fi_lter Command:"), table, 7);
     mcw->mb_data.pop3.filter_cmd =
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, 7, "procmail -f -", label);
 
     advanced =
@@ -1250,6 +1253,14 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
     GtkWidget *entry;
     gint row = -1;
 
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
+
     notebook = gtk_notebook_new();
     table = libbalsa_create_table(8, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), 12);
@@ -1257,16 +1268,16 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = libbalsa_create_label(_("Mailbox _Name:"), table, ++row);
+    label = libbalsa_create_label(_("Mailbox _name:"), table, ++row);
     mcw->mailbox_name =  libbalsa_create_entry(table,
-                                      GTK_SIGNAL_FUNC(check_for_blank_fields),
+                                      G_CALLBACK(check_for_blank_fields),
                                       mcw, row, NULL, label);
 
     /* imap server */
     label = libbalsa_create_label(_("_Server:"), table, ++row);
     mcw->mb_data.imap.bsc.server = 
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, row, "localhost", label);
     mcw->mb_data.imap.bsc.default_ports = IMAP_DEFAULT_PORTS;
 
@@ -1274,7 +1285,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
     label = libbalsa_create_label(_("_Username:"), table, ++row);
     mcw->mb_data.imap.username = 
 	libbalsa_create_entry(table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     G_CALLBACK(check_for_blank_fields),
 		     mcw, row, g_get_user_name(), label);
 
     /* toggle for anonymous password */
@@ -1285,7 +1296,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                      G_CALLBACK(anon_toggle_cb), mcw);
     /* toggle for remember password */
     mcw->mb_data.imap.remember = 
-	libbalsa_create_check(_("_Remember Password"), table, ++row,
+	libbalsa_create_check(_(remember_password_message), table, ++row,
 		     FALSE);
     g_signal_connect(G_OBJECT(mcw->mb_data.imap.remember), "toggled",
                      G_CALLBACK(remember_toggle_cb), mcw);
@@ -1296,7 +1307,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
 	libbalsa_create_entry(table, NULL, NULL, row, NULL, label);
     gtk_entry_set_visibility(GTK_ENTRY(mcw->mb_data.imap.password), FALSE);
 
-    label = libbalsa_create_label(_("F_older Path:"), table, ++row);
+    label = libbalsa_create_label(_("F_older path:"), table, ++row);
 
     mcw->mb_data.imap.folderpath = entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(mcw->mb_data.imap.folderpath), "INBOX");
