@@ -22,9 +22,9 @@
 
 #define _XOPEN_SOURCE 500
 
+#include <stdlib.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
-#include <gnome.h>
 #include <glib/gi18n.h>
 #include "balsa-app.h"
 #include "save-restore.h"
@@ -389,6 +389,18 @@ pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
 
 /* Initialize the specified mailbox, creating the internal data
    structures which represent the mailbox. */
+static void
+sr_special_notify(gpointer data, GObject * mailbox)
+{
+    LibBalsaMailbox **special = data;
+
+    if (special == &balsa_app.trash && !balsa_app.mblist_tree_store
+        && balsa_app.empty_trash_on_exit)
+        empty_trash(balsa_app.main_window);
+
+    *special = NULL;
+}
+
 static gint
 config_mailbox_init(const gchar * prefix)
 {
@@ -441,7 +453,8 @@ config_mailbox_init(const gchar * prefix)
 	     * and outbox, and view->subscribe gets set correctly for
              * trashbox. */
 	    *special = mailbox;
-	    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) special);
+            g_object_weak_ref(G_OBJECT(mailbox),
+                              (GWeakNotify) sr_special_notify, special);
 	}
 
         balsa_mblist_mailbox_node_append(NULL, mbnode);
@@ -488,6 +501,8 @@ config_load_smtp_server(const gchar * key, const gchar * value, gpointer data)
 
     libbalsa_conf_push_group(key);
     smtp_server = libbalsa_smtp_server_new_from_config(value);
+    libbalsa_server_connect_signals(LIBBALSA_SERVER(smtp_server),
+				    G_CALLBACK(ask_password), NULL);
     libbalsa_conf_pop_group();
     libbalsa_smtp_server_add_to_list(smtp_server, smtp_servers);
 
