@@ -22,7 +22,6 @@
 #include "config.h"
 
 #include <string.h>
-#include <gnome.h>
 #include <glib/gi18n.h>
 
 #include "balsa-app.h"
@@ -33,11 +32,22 @@
 #include "toolbar-factory.h"
 #include "toolbar-prefs.h"
 
+#if !GTK_CHECK_VERSION(2, 14, 0)
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
+
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #endif
 
+#ifdef HAVE_GNOME
 #define OLD_BALSA_COMPATIBILITY_TRANSLATION
+#endif
+#ifdef OLD_BALSA_COMPATIBILITY_TRANSLATION
+#include <gnome.h> /* for GNOME_STOCK_* pixmaps */
+#endif
 
 /* Enumeration for GtkTreeModel columns. */
 enum {
@@ -145,19 +155,22 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
 
     /* The order of pages must be consistent with the BalsaToolbarType
      * enum. */
-    model = balsa_window_get_toolbar_model(&ui_manager);
+    model = balsa_window_get_toolbar_model();
+    ui_manager = balsa_window_ui_manager_new(NULL);
     child = create_toolbar_page(model, ui_manager);
     g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
                              gtk_label_new(_("Main window")));
 
-    model = sendmsg_window_get_toolbar_model(&ui_manager);
+    model = sendmsg_window_get_toolbar_model();
+    ui_manager = sendmsg_window_ui_manager_new(NULL);
     child = create_toolbar_page(model, ui_manager);
     g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
                              gtk_label_new(_("Compose window")));
 
-    model = message_window_get_toolbar_model(&ui_manager);
+    model = message_window_get_toolbar_model();
+    ui_manager = message_window_ui_manager_new(NULL);
     child = create_toolbar_page(model, ui_manager);
     g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
@@ -375,7 +388,9 @@ current_selection_changed_cb(GtkTreeSelection * selection, ToolbarPage * page)
 static void
 tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
 {
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
     GError *err = NULL;
+#endif
 
     switch (response) {
     case GTK_RESPONSE_DELETE_EVENT:
@@ -383,13 +398,20 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
         gtk_widget_destroy(GTK_WIDGET(dialog));
         break;
     case GTK_RESPONSE_HELP:
+#if GTK_CHECK_VERSION(2, 14, 0) || HAVE_GNOME
+#if GTK_CHECK_VERSION(2, 14, 0)
+        gtk_show_uri(NULL, "ghelp:balsa?toolbar-prefs",
+                     gtk_get_current_event_time(), &err);
+#else                           /* GTK_CHECK_VERSION(2, 14, 0) */
         gnome_help_display("balsa", "toolbar-prefs", &err);
+#endif                          /* GTK_CHECK_VERSION(2, 14, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
 		    _("Error displaying toolbar help: %s\n"),
 		    err->message);
             g_error_free(err);
         }
+#endif
         break;
     default:
         break;
